//  Panasonic製シーリングライト用赤外線リモコンです。
//　AVR　ATmega88用
//　Programming　By　亀屋　2016/2/01~
//  v1.0.0 2016/2/7　1st Release
//  v1.0.1 2016/3/3  PWR_DOWN at sleep & No WDT

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/wdt.h>

// CPU freq.
#define FOSC 1000000 	// 1MHz internal CPU clock

//PWM cycl.
#define TIM1CYC (FOSC/38460)-1		//PWM周期
#define TIM1MOD (TIM1CYC-1)*2/3		//LED点灯Duty1/3
#define TIM1STO TIM1CYC+2			//PWM STOP
//AEHA format para.
#define	leader_Hi	3500
#define	leader_Lo	1700
#define	signal_Hi	435
#define	signal_1_Lo	1305
#define	signal_0_Lo	435
#define	Txbyte		5
//on ROM (control data:MSB first)
const uint8_t pana1[]={	0x34,0x4A,0x90,0x34,0xA4,		//Pana ch1  On
						0x34,0x4A,0x90,0x74,0xE4,		//Pana ch1  常夜灯
						0x34,0x4A,0x90,0x54,0xC4,		//Pana ch1  Bright
						0x34,0x4A,0x90,0xD4,0x44,		//pana ch1  Dark
						0x34,0x4A,0x90,0xF4,0x64	};	//Pana ch1  Off

const uint8_t pana2[]={	0x34,0x4A,0x90,0x2C,0xBC,		//Pana ch2  On
						0x34,0x4A,0x90,0x6C,0xFC,		//Pana ch2  常夜灯
						0x34,0x4A,0x90,0x4C,0xDC,		//Pana ch2  Bright
						0x34,0x4A,0x90,0xCC,0x5C,		//pana ch2  Dark
						0x34,0x4A,0x90,0xEC,0x7C	};	//Pana ch2  Off
//on RAM
volatile uint8_t SW_in;
volatile uint8_t SW_mode;
volatile uint8_t i,j;
volatile uint8_t TxData[Txbyte*8+8];	//展開後の送信データ0/1格納用

//PIN変化割り込み
ISR (PCINT1_vect)
{
    //wakeup & read key-sw
	SW_in=(PINC & 0x0f);	//PCINT8～11読み取り
}


//LED flicker.
void FLICKx(uint8_t ix)
{
	uint8_t i;
	for(i=0;i<ix;i++)
	{
		PORTC |= _BV(PC5);  //LED消灯
		_delay_ms(50);
		PORTC &= ~_BV(PC5); //LED点灯
		_delay_ms(50);
	}
}

int main(void)
{
	
	//ポート設定
	DDRB=0x3f; PORTB=0x00;					// --ooooOo　PB1(OC1A:Timer1 out)
	DDRC=0x30; PORTC=0x0F;					// --OoRRRR	(SW入力PCIN8～11:P-UP) LED
	DDRD=0xfe; PORTD=0x01;					// oooooooR（UART　Rx:P-UP）
	//ピン変化割り込み設定
	PCICR=0x02;								//PCINT1 許可
	PCMSK1=0x0f;							//PCINT8～11 許可
	//Timer1設定(16bit)
	TCNT1=0x0000;		//カウンタ値クリア
	TCCR1A=0xC2;		//TCNT1<OCR1A=Low出力(反転出力)
	TCCR1B=0x19;		//高速PWM clkI/O (前置分周なし)
	TCCR1C=0x00;		//未使用
	OCR1A=TIM1STO;		//初期値(StopPWM=Low)
	OCR1B=0x0000;		//未使用
	ICR1=TIM1CYC;		//38.46KHz
	TIMSK1=0x00;		//割り込みなし
	
	wdt_disable();		//WDT 停止(省電力のため)

	PRR=0xe7;			//電力削減レジスタ(Timer1のみ起動)
	set_sleep_mode (SLEEP_MODE_PWR_DOWN); //CLK全停止省電力モード
	sleep_enable();		//スリープ許可
	
	sei();				//割り込み許可
	
	FLICKx(3);			//起動確認：LED点灯x3


	for(;;)
	{
		OCR1A=TIM1STO;				//変調STOP(Low)
		PRR=0xef;					//電力削減レジスタ(Timer1も停止)
		sleep_cpu();				//CPUアイドルスリープ

		//SW入力(割り込みでWakeUp)
		PRR=0xe7;					//電力削減レジスタ(Timer1のみ起動)
		switch(SW_in)
		{
			case(0x0e):	//SW1(N.C.)
				SW_mode=1;
			break;
			case(0x0d):	//SW2(下)
				SW_mode=4;
			break;
			case(0x0b):	//SW3(中)
				SW_mode=1;
			break;
			case(0x07):	//SW4(上)
				SW_mode=0;
			break;
			case(0x0f):	//入力なし
				SW_mode=5;
			break;
			default:	//複数押下時
				SW_mode=5;
			break;
		}
		if(SW_mode < 5){	//Key制限
		
		//5byteを0/1にbit展開
		for(i=0;i<Txbyte;i++){
			for(j=0;j<8;j++){
				if(pana1[SW_mode*Txbyte+i] & (0x80>>j)){
					TxData[i*8+j]=1;
				}
				else{
					TxData[i*8+j]=0;
				}
			}
		}

	FLICKx(1);			//起動確認：LED点灯x1

		for(j=0;j<2;j++){	//2送信で1回分

		//送受信開始
		OCR1A = TIM1MOD;		// Leader High
        _delay_us(leader_Hi);
        OCR1A = TIM1STO;		// Leader Low
        _delay_us(leader_Lo);
		//データ部5Byte送信
		for(i=0;i<(Txbyte*8);i++){
			OCR1A = TIM1MOD;	// Data High
			_delay_us(signal_Hi);
			OCR1A = TIM1STO;	// Data Low
			if(TxData[i]==0){
				_delay_us(signal_0_Lo);		//0 1T wait
			}
			else{
				_delay_us(signal_1_Lo);		//1 3T wait
			}
		}
		OCR1A = TIM1MOD;		// Trailer High
        _delay_us(signal_Hi);
        OCR1A = TIM1STO;		// Trailer Low
        _delay_ms(10);

		}
		}
	}
}

